@@ -1,5 +1,5 @@
 // Client.java
-// $Id: Client.java,v 1.49 1999/02/19 16:02:12 ylafon Exp $
+// $Id: Client.java,v 1.52 1999/05/05 14:13:55 ylafon Exp $
 // (c) COPYRIGHT MIT and INRIA, 1996.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -255,10 +255,10 @@ public abstract class Client implements EventHandler {
 	cont = false; // reinit the continue
 	try {
 	    request = (Request) parser.parse();
-	} catch (HttpParserException ex) {
+	} catch (IOException ex) {
 	    // The connection has probably been closed prematurely:
 	    return null;
-	} catch (IOException ex) {
+	} catch (HttpParserException ex) {
 	    if ( debug ) {
 		System.out.println("+++ "+this+" got exception:");
 		ex.printStackTrace();
@@ -438,18 +438,48 @@ public abstract class Client implements EventHandler {
 	alive_loop:
 	    while ( (! interrupted)  && keep ) {
 		// Get the next available request, and  mark client as used:
-		if ( processed == 0 ) {
-		    // Always run for the first request, update client's infos
-		    if ((request = getNextRequest()) == null ) 
-			break alive_loop;
-		    major = request.getMajorVersion();
-		    minor = request.getMinorVersion();
-		} else {
-		    if ( interrupted = idleConnection() )
-			break alive_loop;
-		    if ((request = getNextRequest()) == null) 
-			break alive_loop;
-		    usedConnection();
+		try {
+		    if ( processed == 0 ) {
+			// Always run for the first request, 
+			// update client's infos
+			if ((request = getNextRequest()) == null ) 
+			    break alive_loop;
+			major = request.getMajorVersion();
+			minor = request.getMinorVersion();
+		    } else {
+			if ( interrupted = idleConnection() )
+			    break alive_loop;
+			if ((request = getNextRequest()) == null) 
+			    break alive_loop;
+			usedConnection();
+		    }
+		} catch (ClientException cex) {
+		    if (cex.ex != null && 
+			(cex.ex instanceof HttpParserException)) {
+			HttpParserException hex = (HttpParserException) cex.ex;
+			if (!hex.hasRequest()) {
+			    break alive_loop;
+			}
+			request = (Request) hex.getRequest();
+			if ( processed == 0 ) {
+			    major = request.getMajorVersion();
+			    minor = request.getMinorVersion();
+			} else
+			    usedConnection();
+			reply = (Reply) request.makeBadRequestReply();
+			reply.setContentLength(0);
+			reply.addConnection("close");
+			// Inital keep alive check, emit the reply:
+			if ( keep ) 
+			    keep = tryKeepConnection(request, reply) ;
+			sent = emitReply(reply) ;
+			// Semi-fake log entry
+			log(request, reply, 0, 0) ;
+			processed++;
+			if ( keep )
+			    output.flush(); 
+		    }
+		    break alive_loop;
 		}
 		// Some traces if required:
 		if ( debuglog )
